@@ -3,7 +3,7 @@ wwwDirectory = function(){
   return(path)
 }
 
-createHTML <- function(directory, styles, dependencies, json, show){
+createHTML <- function(directory, styles, dependencies, json){
   if(file.exists(directory))
     unlink(directory, recursive = TRUE)
   dir.create(directory)
@@ -36,9 +36,6 @@ createHTML <- function(directory, styles, dependencies, json, show){
     json <- json()
   html <- sub("<!--json-->",paste0('<script type="application/json" id="data">',json,'</script>'),html)
   write(html, paste(directory, "index.html", sep = "/"))
-  
-  if(identical(show,TRUE))
-    browseURL(normalizePath(paste(directory, "index.html", sep = "/")))
 }
 
 
